# Test Kits

A collection of testing frameworks all collected together into one.

- [Specta](https://github.com/petejkim/specta), a lightweight TDD / BDD
  framework for Objective-C and Cocoa (v0.2.1).
- [Expecta](https://github.com/petejkim/expecta/), a matcher framework for
  Objective-C and Cocoa (v0.2.3-19) - which now has XCTest support.
- [OCMock](http://ocmock.org/) An Objective-C implementation of Mock Objects (2.2.1)

All of these work well with OCUnit, and are easy to set up.

## Instructions

These work for me instead of using CocoaPods

### Set up

I don't add these as submodules to a project. I download and build them
separately and then copy the code/headers/libraries into the project where I
want to use them.

1. Clone the repository locally (with the --recursive option to pull down the submodules as
well)
2. `cd` to the specta folder and run `rake`.
3. `cd` to the expecta folder and run `rake`.
4. No set up needs to be done for OCMock; it's not a submodule and it's already built.

### For iOS

1. Create a subfolders of the tests folder called `Specta` `Expecta`, and
`OCMock`.
2. Copy all the headers from the `specta/products` folder to this `Specta` folder.
3. Copy and add `libSpecta-ios-universal.a` in the `specta/products` folder to this
`Specta` folder.
4. Copy all the headers from the `expecta/products` folder to this `Expecta` folder.
5. Copy and add `libExpecta-ios-universal.a` in the `expecta/products` folder to
this `Expecta` folder.
6. Copy the contents of the `OCMock/iOS` folder to the this `OCMock` folder.
7. From Xcode - add the `Specta`, `Expecta` and `OCMock` folders to the test
target.
8. For the Tests target add `"$(SRCROOT)/<tests-folder>/OCMock"` to the Header
Search Paths.
9. For the Tests target's Other Linker Flags make sure these three lines are
there. (**note** All three are required for OCMock, `-ObjC` is required for
Expecta and none are required for Specta):

````
    -force_load
    "$(SRCROOT)/<tests-folder>/OCMock/libOCMock.a"
    -ObjC
````


### For OS X

1. Create a subfolders of the tests folder called `Specta` and `Expecta`.
2. Copy and add `libSpecta-macosx.a` in the `specta/products` folder to this
`Specta` folder.
3. Copy and add `libExpecta-macosx.a` in the `expecta/products` folder to
this `Expecta` folder.
4. In the Tests target's Other Linker Flags add `-ObjC`.
5. From Xcode - add the `Specta`, and `Expecta` folders to the test
target.
6. From the `OCMock/OSX` folder drag the `OCMock.framework` to the Xcode - under
the Frameworks section - make sure that it applies to the tests target.
7. Create a new Copy Files build phase to the test target. Add the
`OCMock.framework` to this phase and set the destination to Frameworks.
8. If you are using Expecta, add `-ObjC` to the Other Linker Flags for the test
target
9. Move this new build phase above the run script phase.
10. In the Tests target's Runpath Search Paths add `@loader_path/../Frameworks`

### Headers

Add these to the Tests-Prefix.pch file

``` objective-c
// Specta
#import "Specta.h"

// Expecta
#define EXP_SHORTHAND
#import "Expecta.h"

// OCMock
#import <OCMock/OCMock.h>
```

### Smoke Tests

How do you check that all these steps have correctly set up these tools?

There is a folder called `Smoke Tests` which contains 2 test source files that can be
added to the test target. One uses Specta and Expecta, the other uses OCMock. They will fail initially. But
after making obvious corrections the tests will pass. This small "Red/Green" test confirms that your test helpers
are correctly installed.

## Licence

Don't look at me, I'm just collecting Open Source projects into a convenient
bundle that I can use. The license texts are in the individual project
folders. All are non-viral.
